@@ -16,8 +16,8 @@
  * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <libxfs.h>
-#include <libxlog.h>
+#include "libxfs.h"
+#include "libxlog.h"
 #include "bmap.h"
 #include "command.h"
 #include "metadump.h"
@@ -221,7 +221,8 @@ write_buf(
 		bp->b_ops->verify_write(bp);
 		if (bp->b_error) {
 			print_warning(
-				"obfuscation corrupted block at bno 0x%llx/0x%x",
+			    "obfuscation corrupted block at %s bno 0x%llx/0x%x",
+				bp->b_ops->name,
 				(long long)bp->b_bn, bp->b_bcount);
 		}
 	}
@@ -418,7 +419,7 @@ valid_bno(
 		return 1;
 	if (agno == (mp->m_sb.sb_agcount - 1) && agbno > 0 &&
 			agbno <= (mp->m_sb.sb_dblocks -
-			 (xfs_drfsbno_t)(mp->m_sb.sb_agcount - 1) *
+			 (xfs_rfsblock_t)(mp->m_sb.sb_agcount - 1) *
 			 mp->m_sb.sb_agblocks))
 		return 1;
 
@@ -529,7 +530,7 @@ struct name_ent {
 	struct name_ent		*next;
 	xfs_dahash_t		hash;
 	int			namelen;
-	uchar_t			name[1];
+	unsigned char		name[1];
 };
 
 #define NAME_TABLE_SIZE		4096
@@ -555,7 +556,7 @@ nametable_clear(void)
  * return a pointer to its entry, otherwise return a null pointer.
  */
 static struct name_ent *
-nametable_find(xfs_dahash_t hash, int namelen, uchar_t *name)
+nametable_find(xfs_dahash_t hash, int namelen, unsigned char *name)
 {
 	struct name_ent	*ent;
 
@@ -572,7 +573,7 @@ nametable_find(xfs_dahash_t hash, int namelen, uchar_t *name)
  * name's new entry, or a null pointer if an error occurs.
  */
 static struct name_ent *
-nametable_add(xfs_dahash_t hash, int namelen, uchar_t *name)
+nametable_add(xfs_dahash_t hash, int namelen, unsigned char *name)
 {
 	struct name_ent	*ent;
 
@@ -593,10 +594,10 @@ nametable_add(xfs_dahash_t hash, int namelen, uchar_t *name)
 #define is_invalid_char(c)	((c) == '/' || (c) == '\0')
 #define rol32(x,y)		(((x) << (y)) | ((x) >> (32 - (y))))
 
-static inline uchar_t
+static inline unsigned char
 random_filename_char(void)
 {
-	static uchar_t filename_alphabet[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+	static unsigned char filename_alphabet[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 						"abcdefghijklmnopqrstuvwxyz"
 						"0123456789-_";
 
@@ -611,7 +612,7 @@ is_orphanage_dir(
 	struct xfs_mount	*mp,
 	xfs_ino_t		dir_ino,
 	size_t			name_len,
-	uchar_t			*name)
+	unsigned char		*name)
 {
 	return dir_ino == mp->m_sb.sb_rootino &&
 			name_len == ORPHANAGE_LEN &&
@@ -630,7 +631,7 @@ static int
 in_lost_found(
 	xfs_ino_t		ino,
 	int			namelen,
-	uchar_t			*name)
+	unsigned char		*name)
 {
 	static xfs_ino_t	orphanage_ino = 0;
 	char			s[24];	/* 21 is enough (64 bits in decimal) */
@@ -671,13 +672,13 @@ static void
 obfuscate_name(
 	xfs_dahash_t	hash,
 	size_t		name_len,
-	uchar_t		*name)
+	unsigned char	*name)
 {
-	uchar_t		*newp = name;
+	unsigned char	*newp = name;
 	int		i;
 	xfs_dahash_t	new_hash = 0;
-	uchar_t		*first;
-	uchar_t		high_bit;
+	unsigned char	*first;
+	unsigned char	high_bit;
 	int		shift;
 
 	/*
@@ -826,16 +827,16 @@ obfuscate_name(
 static int
 flip_bit(
 	size_t		name_len,
-	uchar_t		*name,
+	unsigned char	*name,
 	uint32_t	bitseq)
 {
 	int	index;
 	size_t	offset;
-	uchar_t	*p0, *p1;
-	uchar_t	m0, m1;
+	unsigned char *p0, *p1;
+	unsigned char m0, m1;
 	struct {
 	    int		byte;	/* Offset from start within name */
-	    uchar_t	bit;	/* Bit within that byte */
+	    unsigned char bit;	/* Bit within that byte */
 	} bit_to_flip[][2] = {	/* Sorted by second entry's byte */
 	    { { 0, 0 }, { 1, 7 } },	/* Each row defines a pair */
 	    { { 1, 0 }, { 2, 7 } },	/* of bytes and a bit within */
@@ -964,7 +965,7 @@ flip_bit(
 static int
 find_alternate(
 	size_t		name_len,
-	uchar_t		*name,
+	unsigned char	*name,
 	uint32_t	seq)
 {
 	uint32_t	bitseq = 0;
@@ -1001,9 +1002,9 @@ find_alternate(
  * are already in the table.
  */
 static int
-handle_duplicate_name(xfs_dahash_t hash, size_t name_len, uchar_t *name)
+handle_duplicate_name(xfs_dahash_t hash, size_t name_len, unsigned char *name)
 {
-	uchar_t		new_name[name_len + 1];
+	unsigned char	new_name[name_len + 1];
 	uint32_t	seq = 1;
 
 	if (!nametable_find(hash, name_len, name))
@@ -1036,7 +1037,7 @@ static void
 generate_obfuscated_name(
 	xfs_ino_t		ino,
 	int			namelen,
-	uchar_t			*name)
+	unsigned char		*name)
 {
 	xfs_dahash_t		hash;
 
@@ -1123,7 +1124,7 @@ process_sf_dir(
 			namelen = ino_dir_size - ((char *)&sfep->name[0] -
 					 (char *)sfp);
 		} else if ((char *)sfep - (char *)sfp +
-				xfs_dir3_sf_entsize(mp, sfp, sfep->namelen) >
+				M_DIROPS(mp)->sf_entsize(sfp, sfep->namelen) >
 				ino_dir_size) {
 			if (show_warnings)
 				print_warning("entry length in dir inode %llu "
@@ -1136,11 +1137,11 @@ process_sf_dir(
 
 		if (obfuscate)
 			generate_obfuscated_name(
-					 xfs_dir3_sfe_get_ino(mp, sfp, sfep),
+					 M_DIROPS(mp)->sf_get_ino(sfp, sfep),
 					 namelen, &sfep->name[0]);
 
 		sfep = (xfs_dir2_sf_entry_t *)((char *)sfep +
-				xfs_dir3_sf_entsize(mp, sfp, namelen));
+				M_DIROPS(mp)->sf_entsize(sfp, namelen));
 	}
 
 	/* zero stale data in rest of space in data fork, if any */
@@ -1160,8 +1161,8 @@ obfuscate_path_components(
 	char			*buf,
 	__uint64_t		len)
 {
-	uchar_t			*comp = (uchar_t *)buf;
-	uchar_t			*end = comp + len;
+	unsigned char		*comp = (unsigned char *)buf;
+	unsigned char		*end = comp + len;
 	xfs_dahash_t		hash;
 
 	while (comp < end) {
@@ -1279,7 +1280,7 @@ process_sf_attr(
 static void
 process_dir_data_block(
 	char		*block,
-	xfs_dfiloff_t	offset,
+	xfs_fileoff_t	offset,
 	int		is_block_format)
 {
 	/*
@@ -1301,7 +1302,7 @@ process_dir_data_block(
 		xfs_dir2_leaf_entry_t	*blp;
 		xfs_dir2_block_tail_t	*btp;
 
-		btp = xfs_dir2_block_tail_p(mp, datahdr);
+		btp = xfs_dir2_block_tail_p(mp->m_dir_geo, datahdr);
 		blp = xfs_dir2_block_leaf_p(btp);
 		if ((char *)blp > (char *)btp)
 			blp = (xfs_dir2_leaf_entry_t *)btp;
@@ -1312,7 +1313,7 @@ process_dir_data_block(
 		else
 			wantmagic = XFS_DIR2_BLOCK_MAGIC;
 	} else { /* leaf/node format */
-		end_of_data = mp->m_dirblkfsbs << mp->m_sb.sb_blocklog;
+		end_of_data = mp->m_dir_geo->fsbcount << mp->m_sb.sb_blocklog;
 		if (xfs_sb_version_hascrc(&mp->m_sb))
 			wantmagic = XFS_DIR3_DATA_MAGIC;
 		else
@@ -1327,9 +1328,9 @@ process_dir_data_block(
 		return;
 	}
 
-	dir_offset = xfs_dir3_data_entry_offset(datahdr);
+	dir_offset = M_DIROPS(mp)->data_entry_offset;
 	ptr = block + dir_offset;
-	endptr = block + mp->m_dirblksize;
+	endptr = block + mp->m_dir_geo->blksize;
 
 	while (ptr < endptr && dir_offset < end_of_data) {
 		xfs_dir2_data_entry_t	*dep;
@@ -1372,7 +1373,7 @@ process_dir_data_block(
 		}
 
 		dep = (xfs_dir2_data_entry_t *)ptr;
-		length = xfs_dir3_data_entsize(mp, dep->namelen);
+		length = M_DIROPS(mp)->data_entsize(dep->namelen);
 
 		if (dir_offset + length > end_of_data ||
 		    ptr + length > endptr) {
@@ -1382,7 +1383,7 @@ process_dir_data_block(
 					(long long)cur_ino);
 			return;
 		}
-		if (be16_to_cpu(*xfs_dir3_data_entry_tag_p(mp, dep)) !=
+		if (be16_to_cpu(*M_DIROPS(mp)->data_entry_tag_p(dep)) !=
 				dir_offset)
 			return;
 
@@ -1395,7 +1396,7 @@ process_dir_data_block(
 		if (zero_stale_data) {
 			/* 1 byte for ftype; don't bother with conditional */
 			int zlen =
-				(char *)xfs_dir3_data_entry_tag_p(mp, dep) -
+				(char *)M_DIROPS(mp)->data_entry_tag_p(dep) -
 				(char *)&dep->name[dep->namelen] - 1;
 			if (zlen > 0) {
 				memset(&dep->name[dep->namelen] + 1, 0, zlen);
@@ -1445,7 +1446,7 @@ add_remote_vals(
 		attr_data.remote_vals[attr_data.remote_val_count] = blockidx;
 		attr_data.remote_val_count++;
 		blockidx++;
-		length -= XFS_LBSIZE(mp);
+		length -= mp->m_sb.sb_blocksize;
 	}
 
 	if (attr_data.remote_val_count >= MAX_REMOTE_VALS) {
@@ -1487,7 +1488,7 @@ process_attr_block(
 	}
 
 	/* Ok, it's a leaf - get header; accounts for crc & non-crc */
-	xfs_attr3_leaf_hdr_from_disk(&hdr, leaf);
+	xfs_attr3_leaf_hdr_from_disk(mp->m_attr_geo, &hdr, leaf);
 
 	nentries = hdr.count;
 	if (nentries * sizeof(xfs_attr_leaf_entry_t) +
@@ -1509,7 +1510,7 @@ process_attr_block(
 		if (!first_name || xfs_attr3_leaf_name(leaf, i) < first_name)
 			first_name = xfs_attr3_leaf_name(leaf, i);
 
-		if (be16_to_cpu(entry->nameidx) > XFS_LBSIZE(mp)) {
+		if (be16_to_cpu(entry->nameidx) > mp->m_sb.sb_blocksize) {
 			if (show_warnings)
 				print_warning(
 				"invalid attr nameidx in inode %llu",
@@ -1577,11 +1578,11 @@ process_attr_block(
 /* Processes symlinks, attrs, directories ... */
 static int
 process_single_fsb_objects(
-	xfs_dfiloff_t	o,
-	xfs_dfsbno_t	s,
-	xfs_dfilblks_t	c,
+	xfs_fileoff_t	o,
+	xfs_fsblock_t	s,
+	xfs_filblks_t	c,
 	typnm_t		btype,
-	xfs_dfiloff_t	last)
+	xfs_fileoff_t	last)
 {
 	char		*dp;
 	int		ret = 0;
@@ -1617,8 +1618,8 @@ process_single_fsb_objects(
 				struct xfs_da3_icnode_hdr hdr;
 				int used;
 
-				xfs_da3_node_hdr_from_disk(&hdr, node);
-				used = xfs_da3_node_hdr_size(node);
+				M_DIROPS(mp)->node_hdr_from_disk(&hdr, node);
+				used = M_DIROPS(mp)->node_hdr_size;
 
 				used += hdr.count
 					* sizeof(struct xfs_da_node_entry);
@@ -1635,11 +1636,11 @@ process_single_fsb_objects(
 		dp = iocur_top->data;
 		switch (btype) {
 		case TYP_DIR2:
-			if (o >= mp->m_dirleafblk)
+			if (o >= mp->m_dir_geo->leafblk)
 				break;
 
 			process_dir_data_block(dp, o,
-						 last == mp->m_dirblkfsbs);
+					 last == mp->m_dir_geo->fsbcount);
 			iocur_top->need_crc = 1;
 			break;
 		case TYP_SYMLINK:
@@ -1675,11 +1676,11 @@ static int mfsb_length;
 
 static int
 process_multi_fsb_objects(
-	xfs_dfiloff_t	o,
-	xfs_dfsbno_t	s,
-	xfs_dfilblks_t	c,
+	xfs_fileoff_t	o,
+	xfs_fsblock_t	s,
+	xfs_filblks_t	c,
 	typnm_t		btype,
-	xfs_dfiloff_t	last)
+	xfs_fileoff_t	last)
 {
 	int		ret = 0;
 
@@ -1694,8 +1695,8 @@ process_multi_fsb_objects(
 	while (c > 0) {
 		unsigned int	bm_len;
 
-		if (mfsb_length + c >= mp->m_dirblkfsbs) {
-			bm_len = mp->m_dirblkfsbs - mfsb_length;
+		if (mfsb_length + c >= mp->m_dir_geo->fsbcount) {
+			bm_len = mp->m_dir_geo->fsbcount - mfsb_length;
 			mfsb_length = 0;
 		} else {
 			mfsb_length += c;
@@ -1722,13 +1723,13 @@ process_multi_fsb_objects(
 			}
 
 			if ((!obfuscate && !zero_stale_data) ||
-			     o >= mp->m_dirleafblk) {
+			     o >= mp->m_dir_geo->leafblk) {
 				ret = write_buf(iocur_top);
 				goto out_pop;
 			}
 
 			process_dir_data_block(iocur_top->data, o,
-					       last == mp->m_dirblkfsbs);
+					       last == mp->m_dir_geo->fsbcount);
 			iocur_top->need_crc = 1;
 			ret = write_buf(iocur_top);
 out_pop:
@@ -1752,11 +1753,11 @@ process_bmbt_reclist(
 	typnm_t			btype)
 {
 	int			i;
-	xfs_dfiloff_t		o, op = NULLDFILOFF;
-	xfs_dfsbno_t		s;
-	xfs_dfilblks_t		c, cp = NULLDFILOFF;
+	xfs_fileoff_t		o, op = NULLFILEOFF;
+	xfs_fsblock_t		s;
+	xfs_filblks_t		c, cp = NULLFILEOFF;
 	int			f;
-	xfs_dfiloff_t		last;
+	xfs_fileoff_t		last;
 	xfs_agnumber_t		agno;
 	xfs_agblock_t		agbno;
 	int			error;
@@ -1822,7 +1823,7 @@ process_bmbt_reclist(
 		}
 
 		/* multi-extent blocks require special handling */
-		if (btype != TYP_DIR2 || mp->m_dirblkfsbs == 1) {
+		if (btype != TYP_DIR2 || mp->m_dir_geo->fsbcount == 1) {
 			error = process_single_fsb_objects(o, s, c, btype, last);
 		} else {
 			error = process_multi_fsb_objects(o, s, c, btype, last);
@@ -1872,8 +1873,8 @@ scanfunc_bmap(
 		xfs_agnumber_t	ag;
 		xfs_agblock_t	bno;
 
-		ag = XFS_FSB_TO_AGNO(mp, be64_to_cpu(pp[i]));
-		bno = XFS_FSB_TO_AGBNO(mp, be64_to_cpu(pp[i]));
+		ag = XFS_FSB_TO_AGNO(mp, get_unaligned_be64(&pp[i]));
+		bno = XFS_FSB_TO_AGBNO(mp, get_unaligned_be64(&pp[i]));
 
 		if (bno == 0 || bno > mp->m_sb.sb_agblocks ||
 				ag > mp->m_sb.sb_agcount) {
@@ -1924,7 +1925,7 @@ process_btinode(
 					    nrecs, itype);
 	}
 
-	maxrecs = xfs_bmdr_maxrecs(mp, XFS_DFORK_SIZE(dip, mp, whichfork), 0);
+	maxrecs = xfs_bmdr_maxrecs(XFS_DFORK_SIZE(dip, mp, whichfork), 0);
 	if (nrecs > maxrecs) {
 		if (show_warnings)
 			print_warning("invalid numrecs (%u) in inode %lld %s "
@@ -1938,8 +1939,8 @@ process_btinode(
 		xfs_agnumber_t	ag;
 		xfs_agblock_t	bno;
 
-		ag = XFS_FSB_TO_AGNO(mp, be64_to_cpu(pp[i]));
-		bno = XFS_FSB_TO_AGBNO(mp, be64_to_cpu(pp[i]));
+		ag = XFS_FSB_TO_AGNO(mp, get_unaligned_be64(&pp[i]));
+		bno = XFS_FSB_TO_AGBNO(mp, get_unaligned_be64(&pp[i]));
 
 		if (bno == 0 || bno > mp->m_sb.sb_agblocks ||
 				ag > mp->m_sb.sb_agcount) {
@@ -2101,7 +2102,7 @@ done:
 		need_new_crc = 1;
 
 	if (crc_was_ok && need_new_crc)
-		xfs_dinode_calc_crc(mp, dip);
+		libxfs_dinode_calc_crc(mp, dip);
 	return success;
 }
 
@@ -2115,13 +2116,43 @@ copy_inode_chunk(
 	xfs_agino_t 		agino;
 	int			off;
 	xfs_agblock_t		agbno;
+	xfs_agblock_t		end_agbno;
 	int			i;
 	int			rval = 0;
+	int			blks_per_buf;
+	int			inodes_per_buf;
+	int			ioff;
 
 	agino = be32_to_cpu(rp->ir_startino);
 	agbno = XFS_AGINO_TO_AGBNO(mp, agino);
+	end_agbno = agbno + mp->m_ialloc_blks;
 	off = XFS_INO_TO_OFFSET(mp, agino);
 
+	/*
+	 * If the fs supports sparse inode records, we must process inodes a
+	 * cluster at a time because that is the sparse allocation granularity.
+	 * Otherwise, we risk CRC corruption errors on reads of inode chunks.
+	 *
+	 * Also make sure that that we don't process more than the single record
+	 * we've been passed (large block sizes can hold multiple inode chunks).
+	 */
+	if (xfs_sb_version_hassparseinodes(&mp->m_sb))
+		blks_per_buf = xfs_icluster_size_fsb(mp);
+	else
+		blks_per_buf = mp->m_ialloc_blks;
+	inodes_per_buf = min(blks_per_buf << mp->m_sb.sb_inopblog,
+			     XFS_INODES_PER_CHUNK);
+
+	/*
+	 * Sanity check that we only process a single buffer if ir_startino has
+	 * a buffer offset. A non-zero offset implies that the entire chunk lies
+	 * within a block.
+	 */
+	if (off && inodes_per_buf != XFS_INODES_PER_CHUNK) {
+		print_warning("bad starting inode offset %d", off);
+		return 0;
+	}
+
 	if (agino == 0 || agino == NULLAGINO || !valid_bno(agno, agbno) ||
 			!valid_bno(agno, XFS_AGINO_TO_AGBNO(mp,
 					agino + XFS_INODES_PER_CHUNK - 1))) {
@@ -2131,21 +2162,10 @@ copy_inode_chunk(
 		return 1;
 	}
 
-	push_cur();
-	set_cur(&typtab[TYP_INODE], XFS_AGB_TO_DADDR(mp, agno, agbno),
-			XFS_FSB_TO_BB(mp, XFS_IALLOC_BLOCKS(mp)),
-			DB_RING_IGN, NULL);
-	if (iocur_top->data == NULL) {
-		print_warning("cannot read inode block %u/%u", agno, agbno);
-		rval = !stop_on_read_error;
-		goto pop_out;
-	}
-
 	/*
 	 * check for basic assumptions about inode chunks, and if any
 	 * assumptions fail, don't process the inode chunk.
 	 */
-
 	if ((mp->m_sb.sb_inopblock <= XFS_INODES_PER_CHUNK && off != 0) ||
 			(mp->m_sb.sb_inopblock > XFS_INODES_PER_CHUNK &&
 					off % XFS_INODES_PER_CHUNK != 0) ||
@@ -2155,29 +2175,45 @@ copy_inode_chunk(
 		if (show_warnings)
 			print_warning("badly aligned inode (start = %llu)",
 					XFS_AGINO_TO_INO(mp, agno, agino));
-		goto skip_processing;
+		return 1;
 	}
 
-	/*
-	 * scan through inodes and copy any btree extent lists, directory
-	 * contents and extended attributes.
-	 */
-	for (i = 0; i < XFS_INODES_PER_CHUNK; i++) {
-		xfs_dinode_t            *dip;
+	push_cur();
+	ioff = 0;
+	while (agbno < end_agbno && ioff < XFS_INODES_PER_CHUNK) {
+		if (xfs_inobt_is_sparse_disk(rp, ioff))
+			goto next_bp;
 
-		dip = (xfs_dinode_t *)((char *)iocur_top->data +
-				((off + i) << mp->m_sb.sb_inodelog));
-
-		/* process_inode handles free inodes, too */
-		if (!process_inode(agno, agino + i, dip,
-				   XFS_INOBT_IS_FREE_DISK(rp, i)))
+		set_cur(&typtab[TYP_INODE], XFS_AGB_TO_DADDR(mp, agno, agbno),
+			XFS_FSB_TO_BB(mp, blks_per_buf), DB_RING_IGN, NULL);
+		if (iocur_top->data == NULL) {
+			print_warning("cannot read inode block %u/%u",
+				      agno, agbno);
+			rval = !stop_on_read_error;
 			goto pop_out;
-	}
-skip_processing:
-	if (write_buf(iocur_top))
-		goto pop_out;
+		}
 
-	inodes_copied += XFS_INODES_PER_CHUNK;
+		for (i = 0; i < inodes_per_buf; i++) {
+			xfs_dinode_t	*dip;
+
+			dip = (xfs_dinode_t *)((char *)iocur_top->data +
+					((off + i) << mp->m_sb.sb_inodelog));
+
+			/* process_inode handles free inodes, too */
+			if (!process_inode(agno, agino + ioff + i, dip,
+			    XFS_INOBT_IS_FREE_DISK(rp, i)))
+				goto pop_out;
+
+			inodes_copied++;
+		}
+
+		if (write_buf(iocur_top))
+			goto pop_out;
+
+next_bp:
+		agbno += blks_per_buf;
+		ioff += inodes_per_buf;
+	}
 
 	if (show_progress)
 		print_progress("Copied %u of %u inodes (%u of %u AGs)",
@@ -2477,7 +2513,12 @@ copy_sb_inodes(void)
 static int
 copy_log(void)
 {
-	int dirty;
+	struct xlog	log;
+	int		dirty;
+	xfs_daddr_t	logstart;
+	int		logblocks;
+	int		logversion;
+	int		cycle = XLOG_INIT_CYCLE;
 
 	if (show_progress)
 		print_progress("Copying log");
@@ -2495,15 +2536,23 @@ copy_log(void)
 	if (!obfuscate && !zero_stale_data)
 		goto done;
 
-	dirty = xlog_is_dirty(mp, &x, 0);
+	dirty = xlog_is_dirty(mp, &log, &x, 0);
 
 	switch (dirty) {
 	case 0:
 		/* clear out a clean log */
 		if (show_progress)
 			print_progress("Zeroing clean log");
-		memset(iocur_top->data, 0,
-			mp->m_sb.sb_logblocks * mp->m_sb.sb_blocksize);
+
+		logstart = XFS_FSB_TO_DADDR(mp, mp->m_sb.sb_logstart);
+		logblocks = XFS_FSB_TO_BB(mp, mp->m_sb.sb_logblocks);
+		logversion = xfs_sb_version_haslogv2(&mp->m_sb) ? 2 : 1;
+		if (xfs_sb_version_hascrc(&mp->m_sb))
+			cycle = log.l_curr_cycle + 1;
+
+		libxfs_log_clear(NULL, iocur_top->data, logstart, logblocks,
+				 &mp->m_sb.sb_uuid, logversion,
+				 mp->m_sb.sb_logsunit, XLOG_FMT, cycle);
 		break;
 	case 1:
 		/* keep the dirty log */
