@@ -16,7 +16,7 @@
  * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <xfs/command.h>
+#include "command.h"
 #include <ctype.h>
 #include <pwd.h>
 #include <grp.h>
